PONTIS_ALL_ASSETS = [
    {"type": "SPOT", "pair": ("BTC", "USD")},
    {"type": "SPOT", "pair": ("ETH", "USD")},
    {"type": "SPOT", "pair": ("LUNA", "USD")},
    {"type": "SPOT", "pair": ("SOL", "USD")},
    {"type": "SPOT", "pair": ("AVAX", "USD")},
    {"type": "SPOT", "pair": ("DOGE", "USD")},
    {"type": "SPOT", "pair": ("SHIB", "USD")},
    {"type": "SPOT", "pair": ("TEMP", "USD")},
    {"type": "SPOT", "pair": ("DAI", "USD")},
    {"type": "SPOT", "pair": ("USDT", "USD")},
    {"type": "SPOT", "pair": ("USDC", "USD")},
    {"type": "SPOT", "pair": ("TUSD", "USD")},
    {"type": "SPOT", "pair": ("ETH", "MXN")},
    {"type": "FUTURE", "pair": ("BTC", "USD")},
    {"type": "FUTURE", "pair": ("ETH", "USD")},
]
